defmodule Mix.Tasks.Pleroma.RefreshUserFields do
  @shortdoc "Refresh user fields: fetches user field contents and re-submits/saves them."

  use Mix.Task
  alias Pleroma.User
  import Ecto.Query

  require Logger

  def run_helper do
    linkify_opts = Pleroma.Config.get(Pleroma.Formatter)

    User
    |> where([u], not fragment("? = '[]'", u.raw_fields))
    |> Pleroma.Repo.chunk_stream(1, :batches)
    |> Stream.each(fn
      [%User{raw_fields: raw_fields} = user] ->
        try do
          valid_fields =
            Enum.map(raw_fields, fn f ->
              Map.update!(f, "value", &Linkify.link(&1, linkify_opts))
            end)

          if user.fields != valid_fields do
            Logger.info(
              "Updating the user (id:#{user.id}), previous fields: #{inspect(user.fields)}, new fields: #{
                inspect(valid_fields)
              }"
            )

            user
            |> User.update_changeset(%{fields: valid_fields})
            |> User.update_and_set_cache()
          end
        rescue
          error ->
            Logger.error("Error updating the user (id:#{user.id}), #{inspect(error)}")
        end

      _ ->
        :noop
    end)
    |> Stream.run()
  end

  def run([]) do
    Mix.Pleroma.start_pleroma()
    run_helper()
  end
end
