defmodule Pleroma.Mfc.Notify do
  alias Pleroma.Activity
  alias Pleroma.User

  use Pleroma.Workers.WorkerHelper, queue: "mfc_notify"

  def user_change(%User{id: id}) do
    if Pleroma.Config.get([:mfc, :enable_user_info_sync]) do
      enqueue("user_change", %{"user_id" => id})
    end
  end

  def account_creation(%User{id: id}) do
    if Pleroma.Config.get([:mfc, :enable_account_creation_sync]) do
      enqueue("account_creation", %{"user_id" => id})
    end
  end

  def sync_follows(%User{id: id}) do
    enqueue("sync_follows", %{"user_id" => id})
  end

  def status_create(%Activity{id: id}) do
    enqueue("status_create", %{"activity_id" => id})
  end

  def status_delete(%Activity{id: activity_id}, %User{id: user_id}, deleted_at) do
    if Pleroma.Config.get([:mfc, :enable_status_deletion_sync]) do
      enqueue("status_delete", %{
        "activity_id" => activity_id,
        "user_id" => user_id,
        "deleted_at" => NaiveDateTime.to_iso8601(deleted_at)
      })
    end
  end

  def perform(%Job{args: %{"op" => "user_change", "user_id" => user_id}}) do
    user_id
    |> User.get_cached_by_id()
    |> Pleroma.Web.Mfc.Api.notify_user_change()
  end

  def perform(%Job{args: %{"op" => "account_creation", "user_id" => user_id}}) do
    user_id
    |> User.get_cached_by_id()
    |> Pleroma.Web.Mfc.Api.notify_account_creation()
  end

  def perform(%Job{args: %{"op" => "sync_follows", "user_id" => user_id}}) do
    user_id
    |> User.get_cached_by_id()
    |> Pleroma.Web.Mfc.Utils.sync_follows()
  end

  def perform(%Job{args: %{"op" => "status_create", "activity_id" => activity_id}}) do
    activity_id
    |> Activity.get_by_id()
    |> Pleroma.Web.Mfc.Api.notify_status_creation()
  end

  def perform(%Job{
        args: %{
          "op" => "status_delete",
          "activity_id" => activity_id,
          "user_id" => user_id,
          "deleted_at" => deleted_at
        }
      }) do
    activity = Activity.get_by_id(activity_id)
    user = User.get_cached_by_id(user_id)
    deleted_at = NaiveDateTime.from_iso8601!(deleted_at)

    Pleroma.Web.Mfc.Api.notify_status_deletion(activity, user, deleted_at)
  end
end
