defmodule Mix.Tasks.Mfc.ReplaceAttachmentUrlPrefixTest do
  use Pleroma.DataCase, async: true
  alias Mix.Tasks.Mfc.ReplaceAttachmentUrlPrefix
  alias Pleroma.Object

  test "replaces urls prefix" do
    Object.create(%{
      "actor" => "http://localhost:4001/users/egor",
      "meta" => %{},
      "name" => "moog-modular-eurorack.jpg",
      "type" => "Document",
      "url" => [
        %{
          "href" =>
            "http://localhost:4001/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg",
          "mediaType" => "image/jpeg",
          "type" => "Link"
        }
      ]
    })

    Object.create(%{
      "actor" => "http://localhost:4001/users/egor",
      "meta" => %{},
      "name" => "Moog_921,_911,_902_modules.jpg",
      "type" => "Document",
      "url" => [
        %{
          "href" =>
            "http://localhost:4001/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg",
          "mediaType" => "image/jpeg",
          "type" => "Link"
        }
      ]
    })

    Object.create(%{
      "actor" => "http://localhost:4001/users/egor",
      "attachment" => [
        %{
          "actor" => "http://localhost:4001/users/egor",
          "meta" => %{},
          "name" => "moog-modular-eurorack.jpg",
          "type" => "Document",
          "url" => [
            %{
              "href" =>
                "http://localhost:4001/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg",
              "mediaType" => "image/jpeg",
              "type" => "Link"
            }
          ]
        },
        %{
          "actor" => "http://localhost:4001/users/egor",
          "meta" => %{},
          "name" => "Moog_921,_911,_902_modules.jpg",
          "type" => "Document",
          "url" => [
            %{
              "href" =>
                "http://localhost:4001/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg",
              "mediaType" => "image/jpeg",
              "type" => "Link"
            }
          ]
        }
      ],
      "cc" => ["http://localhost:4001/users/egor/followers"],
      "content" =>
        "​<br><a href=\"http://localhost:4001/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg\" class='attachment'>moog-modular-eurorack.jpg</a><br><a href=\"http://localhost:4001/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg\" class='attachment'>Moog_921,_911,_902_modules.jpg…</a>",
      "context" => "http://localhost:4001/contexts/5b6962d6-266c-4f5b-8197-9873d224503e",
      "context_id" => 73,
      "emoji" => %{},
      "id" => "http://localhost:4001/objects/3bd8779e-2dd4-4265-a96e-799c43dce013",
      "published" => "2019-07-31T10:22:05.602466Z",
      "sensitive" => false,
      "summary" => "",
      "tag" => [],
      "to" => ["https://www.w3.org/ns/activitystreams#Public"],
      "type" => "Note"
    })

    ReplaceAttachmentUrlPrefix.process("http://localhost:4001", "https://example.com")

    assert [obj1, obj2, obj3] = Pleroma.Repo.all(Object)

    assert [
             %{
               "href" =>
                 "https://example.com/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg"
             }
           ] = obj1.data["url"]

    assert [
             %{
               "href" =>
                 "https://example.com/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg"
             }
           ] = obj2.data["url"]

    assert [att1, att2] = obj3.data["attachment"]

    assert [
             %{
               "href" =>
                 "https://example.com/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg"
             }
           ] = att1["url"]

    assert [
             %{
               "href" =>
                 "https://example.com/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg"
             }
           ] = att2["url"]

    assert obj3.data["content"] ==
             "​<br><a href=\"https://example.com/media/5aaf4e7b78da66817d092bb15bb8242744cfb545626d384d0ecdda0f00c4815b.jpg?name=moog-modular-eurorack.jpg\" class='attachment'>moog-modular-eurorack.jpg</a><br><a href=\"https://example.com/media/78cd76ab5c95d334da275e93351d716a670b8b7d69759c1eb578343b5df28359.jpg?name=Moog_921%2C_911%2C_902_modules.jpg\" class='attachment'>Moog_921,_911,_902_modules.jpg…</a>"
  end
end
