# Pleroma: A lightweight social networking server
# Copyright © 2017-2019 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule PleromaMfc.Web.MastodonAPI.AccountControllerTest do
  use Pleroma.Web.ConnCase

  import Pleroma.Factory

  test "it sets mfc_follower_sync" do
    user = insert(:user, mfc_follower_sync: "false")

    [conn: conn] =
      oauth_access(["write:accounts"], user: user)
      |> request_content_type()

    conn
    |> patch("/api/v1/accounts/update_credentials", %{mfc_follower_sync: "true"})
    |> json_response_and_validate_schema(200)

    assert refresh_record(user).mfc_follower_sync == true
  end

  test "it un-sets mfc_follower_sync" do
    user = insert(:user, mfc_follower_sync: "true")

    [conn: conn] =
      oauth_access(["write:accounts"], user: user)
      |> request_content_type()

    conn
    |> patch("/api/v1/accounts/update_credentials", %{mfc_follower_sync: "false"})
    |> json_response(200)

    assert refresh_record(user).mfc_follower_sync == false
  end

  test "it doesn't lock an account if it's an mfc model" do
    env = %{user: user} = oauth_access(["write:accounts"])
    [conn: conn] = request_content_type(env)

    conn
    |> patch("/api/v1/accounts/update_credentials", %{locked: "true"})
    |> json_response(200)

    assert refresh_record(user).is_locked == true

    user1 = insert(:user, %{tags: ["mfc_model"]})
    env = oauth_access(["write:accounts"], user: user1)
    [conn: conn] = request_content_type(env)

    conn
    |> patch("/api/v1/accounts/update_credentials", %{locked: "true"})
    |> json_response(200)

    refute refresh_record(user1).is_locked == true
  end
end
