# Pleroma: A lightweight social networking server
# Copyright © 2017-2019 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule PleromaMfc.Web.MastodonAPI.MastodonAPIControllerTest do
  use Pleroma.Web.ConnCase

  import Tesla.Mock

  setup do
    mock(fn env -> apply(HttpRequestMock, :request, [env]) end)
    :ok
  end

  describe "media upload" do
    setup do
      %{conn: conn, user: user} = oauth_access(["write:media"])

      image = %Plug.Upload{
        content_type: "image/jpg",
        path: Path.absname("test/fixtures/image.jpg"),
        filename: "an_image.jpg"
      }

      [conn: conn, user: user, image: image]
    end

    setup do: clear_config([Pleroma.Upload, :uploader])
    setup do: clear_config([:media_proxy])
    setup do: clear_config([Pleroma.Uploaders.MFC, :image_conversion, :postfix_preview_name])

    test "returns a preview url on an MFC image upload", %{conn: conn, image: image} do
      Pleroma.Config.put([Pleroma.Upload, :uploader], Pleroma.Uploaders.MFC)

      mock(fn
        %{method: :post, url: "http://test.test/api/v2/images"} ->
          json(%{
            versions: %{original: %{width: 2048, height: 2048}, small: %{width: 800, height: 800}}
          })
      end)

      image_config = [Pleroma.Uploaders.MFC, :image_conversion, :postfix_preview_name]
      Pleroma.Config.put(image_config, ".preview.jpg")

      %{"preview_url" => preview_url, "url" => url} =
        conn
        |> put_req_header("content-type", "multipart/form-data")
        |> post("/api/v1/media", %{"file" => image})
        |> json_response(:ok)

      assert Path.basename(url) == image.filename
      assert Path.basename(preview_url) == "#{image.filename}.preview.jpg"
    end
  end
end
