# Pleroma: A lightweight social networking server
# Copyright © 2017-2018 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule PleromaMfc.Web.MastodonAPI.StatusViewTest do
  use Pleroma.DataCase

  alias Pleroma.Web.MastodonAPI.StatusView
  import Tesla.Mock

  setup do
    mock(fn env -> apply(HttpRequestMock, :request, [env]) end)
    :ok
  end

  @meta %{
    "original" => %{
      "width" => 2048,
      "height" => 2048,
      "size" => "2048x2048",
      "aspect" => 1.2484394506866416,
      "content_type" => "image/png",
      "dest_key" => "original.png",
      "method" => "resize"
    },
    "small" => %{
      "width" => 800,
      "height" => 800,
      "size" => "800x800",
      "aspect" => 1.2484394506866416,
      "content_type" => "image/png",
      "dest_key" => "small.png",
      "method" => "smartcrop"
    }
  }

  setup do: clear_config([Pleroma.Upload, :uploader])

  test "MFC/attachments" do
    Pleroma.Config.put([Pleroma.Upload, :uploader], Pleroma.Uploaders.MFC)

    object = %{
      "type" => "Image",
      "url" => [
        %{
          "mediaType" => "image/png",
          "href" => "jimi-hendrix.png"
        }
      ],
      "uuid" => 6,
      "meta" => @meta
    }

    expected = %{
      id: "-1312277582",
      type: "image",
      url: "jimi-hendrix.png",
      remote_url: "jimi-hendrix.png",
      preview_url: "jimi-hendrix.png.preview.jpg",
      text_url: "jimi-hendrix.png",
      description: nil,
      meta: @meta,
      pleroma: %{mime_type: "image/png"},
      blurhash: nil
    }

    assert expected == StatusView.render("attachment.json", %{attachment: object, local: true})
    refute expected == StatusView.render("attachment.json", %{attachment: object})

    # If theres a "id", use that instead of the generated one
    object = Map.put(object, "id", 2)
    assert %{id: "2"} = StatusView.render("attachment.json", %{attachment: object, local: true})
  end
end
