# Pleroma: A lightweight social networking server
# Copyright © 2017-2019 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule PleromaMfc.Web.PleromaAPI.AccountControllerTest do
  use Pleroma.Web.ConnCase

  describe "PATCH /api/v1/pleroma/accounts/update_nickname" do
    setup do: oauth_access(["write:accounts"])

    setup do: clear_config([:instance, :federating], false)

    setup do: clear_config([:instance, :allow_local_nickname_changing], true)

    test "local user's nickname can be changed", %{user: user, conn: conn} do
      new_nickname = user.nickname <> "a"

      conn =
        conn
        |> put_req_header("content-type", "application/json")
        |> patch("/api/v1/pleroma/accounts/update_nickname", %{nickname: new_nickname})

      assert json_response_and_validate_schema(conn, 200)

      user = refresh_record(user)
      assert new_nickname == user.nickname
    end

    test "renders error json if nickname validation failed, extended nickname format enabled", %{
      conn: conn
    } do
      clear_config([:instance, :extended_nickname_format], true)

      conn =
        conn
        |> put_req_header("content-type", "application/json")
        |> patch("/api/v1/pleroma/accounts/update_nickname", %{nickname: "^bad%nickname"})

      assert json_response_and_validate_schema(conn, 422) == %{
               "error" => "nickname has invalid format"
             }
    end

    test "renders error json if nickname validation failed, extended nickname format disabled", %{
      conn: conn
    } do
      clear_config([:instance, :extended_nickname_format], false)

      conn =
        conn
        |> put_req_header("content-type", "application/json")
        |> patch("/api/v1/pleroma/accounts/update_nickname", %{nickname: "^bad%nickname"})

      assert json_response_and_validate_schema(conn, 422) == %{
               "error" => "nickname has invalid format"
             }
    end
  end
end
