# Pleroma: A lightweight social networking server
# Copyright © 2017-2019 Pleroma Authors <https://pleroma.social/>
# SPDX-License-Identifier: AGPL-3.0-only

defmodule PleromaMfc.Web.TwitterAPI.UtilControllerTest do
  use Pleroma.Web.ConnCase
  use Oban.Testing, repo: Pleroma.Repo

  alias Pleroma.User

  import Pleroma.Factory

  describe "POST /api/pleroma/disable_account" do
    setup do
      user = insert(:user, password_hash: nil)
      oauth_access(["write:accounts"], user: user)
    end

    test "it checks for confirmation text for MFC users", %{user: user, conn: conn} do
      response =
        conn
        |> post("/api/pleroma/disable_account", %{"confirmation" => "I AGREE"})
        |> json_response(:ok)

      assert response == %{"status" => "success"}

      task = %Oban.Job{args: %{"op" => "deactivate_user", "user_id" => user.id, "status" => true}}

      assert Pleroma.Workers.BackgroundWorker.perform(task)

      user = User.get_cached_by_id(user.id)
      assert user.deactivated == true
    end

    test "it returns 400 error when confirmation text is wrong", %{user: user, conn: conn} do
      conn = post(conn, "/api/pleroma/disable_account", %{"confirmation" => "NO WAY"})

      assert json_response(conn, 400) == %{"error" => "Confirmation text is not valid"}

      refute_enqueued(
        worker: Pleroma.Workers.BackgroundWorker,
        args: %{"op" => "deactivate_user", "user_id" => user.id, "status" => true}
      )

      user = User.get_cached_by_id(user.id)
      assert user.deactivated == false
    end
  end
end
